/*
 * Copyright (c) 2016, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.msf4j.util;

import java.nio.Buffer;
import java.nio.ByteBuffer;
import java.util.List;

/**
 * Utility methods for buffer operations.
 */
public class BufferUtil {

    /**
     * Merge a list of buffers into a single buffer.
     *
     * @param byteBuffers list of ByteBuffer objects
     * @return merged ByteBuffer
     */
    public static ByteBuffer merge(List<ByteBuffer> byteBuffers) {
        if (byteBuffers == null || byteBuffers.size() == 0) {
            return ByteBuffer.allocate(0);
        } else if (byteBuffers.size() == 1) {
            return byteBuffers.get(0);
        } else {
            ByteBuffer fullContent = ByteBuffer.allocate(
                    byteBuffers.stream()
                            .mapToInt(Buffer::capacity)
                            .sum()
            );
            byteBuffers.forEach(fullContent::put);
            fullContent.flip();
            return fullContent;
        }
    }

}
